#ifndef ARRAY_H
#define ARRAY_H

#include <stdlib.h>

#include "buffer.h"

#define DATA_IS_STRING(x) (x->type == TYPE_STRING)

typedef enum { TYPE_UNSET, TYPE_STRING, TYPE_COUNT, TYPE_ARRAY, TYPE_INTEGER, TYPE_FASTCGI } data_type_t;
#define DATA_UNSET \
	data_type_t type; \
	buffer *key; \
	void (* free)(struct data_unset *p); \
	void (* reset)(struct data_unset *p); \
	int (*insert_dup)(struct data_unset *dst, struct data_unset *src); \
	void (*print)(struct data_unset *p)

typedef struct data_unset {
	DATA_UNSET;
} data_unset;

typedef struct {
	data_unset  **data;
	
	size_t *sorted;
	
	size_t used;
	size_t size;
	
	size_t next_power_of_2;
} array;

typedef struct {
	DATA_UNSET;
	
	int count;
} data_count;

data_count *data_count_init();

typedef struct {
	DATA_UNSET;
	
	buffer *value;
} data_string;

data_string *data_string_init();

typedef struct {
	DATA_UNSET;
	
	array *value;
} data_array;

data_array *data_array_init();

typedef struct {
	DATA_UNSET;
	
	int value;
} data_integer;

data_integer *data_integer_init();

typedef struct {
	DATA_UNSET;
	
	buffer *host;
	buffer *docroot;
	unsigned short port;
	
	ssize_t usage;
	time_t disable_ts;
} data_fastcgi;

data_fastcgi *data_fastcgi_init();

array *array_init();
void array_free(array *a);
void array_reset(array *a);
int array_insert_unique(array *a, data_unset *str);
int array_print(array *a);
int array_get_index(array *a, const char *key, int *ndx);
data_unset *array_get_unused_element(array *a, data_type_t t);
data_unset *array_get_element(array *a, const char *key);

#endif
