#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#include "base.h"
#include "log.h"
#include "buffer.h"

#include "plugin.h"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

typedef struct {
	array *conf_access_deny;
} plugin_data;

INIT_FUNC(mod_access_init) {
	plugin_data *p;
	
	p = malloc(sizeof(plugin_data));
	memset(p, 0, sizeof(plugin_data));
	
	p->conf_access_deny = array_init();
	
	return p;
}

FREE_FUNC(mod_access_free) {
	plugin_data *p = p_d;
	
	if (!p) return HANDLER_GO_ON;
	
	array_free(p->conf_access_deny);
	
	free(p);
	
	return HANDLER_GO_ON;
}

SETDEFAULTS_FUNC(mod_access_set_defaults) {
	plugin_data *p = p_d;
	
	config_values_t cv[] = { 
		{ "url.access-deny",             NULL, T_CONFIG_ARRAY },
		{ NULL,                          NULL, T_CONFIG_UNSET }
	};
	
	if (!p) return HANDLER_ERROR;
	
	cv[0].destination = p->conf_access_deny;
	
	if (config_insert_values(srv, cv)) {
		return HANDLER_ERROR;
	}
	
	return HANDLER_GO_ON;
}


handler_t mod_access_uri_handler(server *srv, connection *con, void *p_data) {
	plugin_data *p = p_data;
	int s_len;
	size_t k;
	
	if (con->uri.path->used == 0) return HANDLER_GO_ON;
	
	s_len = con->uri.path->used - 1;
	
	for (k = 0; k < p->conf_access_deny->used; k++) {
		data_string *ds = (data_string *)p->conf_access_deny->data[k];
		int ct_len = ds->value->used - 1;
		
		if (ct_len > s_len) continue;
		if (ds->value->used == 0) continue;
		
		if (0 == strncmp(con->uri.path->ptr + s_len - ct_len, ds->value->ptr, ct_len)) {
			con->http_status = 403;
	
			return HANDLER_FINISHED;
		}
	}
	
	/* not found */
	return HANDLER_GO_ON;
}


int mod_access_plugin_init(plugin *p) {
	p->name        = buffer_init_string("access");
	
	p->init        = mod_access_init;
	p->set_defaults = mod_access_set_defaults;
	p->handle_uri_clean  = mod_access_uri_handler;
	p->cleanup     = mod_access_free;
	
	p->data        = NULL;
	
	return 0;
}
